@@ -60,9 +60,16 @@ role {
 		my $stage        = $self->_stages->{$name};
 		$stage->_analyse_completion_state($finished_job);
 		if ($stage->is_fail
-				&& $stage->_has_should_choose_retry
 				&& $stage->_chosen_retries < $stage->choose_retries
-				&& $stage->should_choose_retry($finished_job->stats,$finished_job->stderr) ) {
+				&& ($finished_job->_analysis_chose_retry
+					|| ( $stage->_has_should_choose_retry
+						&& $stage->should_choose_retry->(
+							$finished_job->stats,
+							$finished_job->_stderr
+						)
+					)
+				)
+			) {
 			$stage->_set_state('retry');
 			$stage->_chosen_retries($stage->_chosen_retries+1);
 			$self->info( "Using user chosen retry: ",
@@ -81,7 +88,9 @@ role {
 			);
 		}
 		my $state = $stage->state;
-		$self->info("stage($name) completion state: $state");
+		$finished_job->stats->{final_job_state} = $state;
+		my $index = $finished_job->index;
+		$self->info("stage($name) run($index) completion state: $state");
 		return $stage;
 	};
 
